@@ -13,13 +13,17 @@ from newrelic.samplers.decorators import (
         data_source_generator as __data_source_generator,
         data_source_factory as __data_source_factory)
 
+from newrelic.api.log import NewRelicContextFormatter
+
 from newrelic.api.application import (
         application_instance as __application,
         register_application as __register_application,
         application_settings as __application_settings)
 
 from newrelic.api.time_trace import (
-        current_trace as __current_trace)
+        current_trace as __current_trace,
+        get_linking_metadata as __get_linking_metadata,
+        add_custom_span_attribute as __add_custom_span_attribute)
 
 from newrelic.api.transaction import (
         current_transaction as __current_transaction,
@@ -42,6 +46,8 @@ from newrelic.api.transaction import (
         record_custom_event as __record_custom_event,
         accept_distributed_trace_payload as __accept_distributed_trace_payload,
         create_distributed_trace_payload as __create_distributed_trace_payload,
+        accept_distributed_trace_headers as __accept_distributed_trace_headers,
+        insert_distributed_trace_headers as __insert_distributed_trace_headers,
         current_trace_id as __current_trace_id,
         current_span_id as __current_span_id)
 
@@ -184,6 +190,10 @@ application_settings = __wrap_api_call(__application_settings,
         'application_settings')
 current_trace = __wrap_api_call(__current_trace,
         'current_trace')
+get_linking_metadata = __wrap_api_call(__get_linking_metadata,
+        'get_linking_metadata')
+add_custom_span_attribute = __wrap_api_call(__add_custom_span_attribute,
+        'add_custom_span_attribute')
 current_transaction = __wrap_api_call(__current_transaction,
         'current_transaction')
 set_transaction_name = __wrap_api_call(__set_transaction_name,
@@ -225,6 +235,12 @@ accept_distributed_trace_payload = __wrap_api_call(
 create_distributed_trace_payload = __wrap_api_call(
         __create_distributed_trace_payload,
         'create_distributed_trace_payload')
+accept_distributed_trace_headers = __wrap_api_call(
+        __accept_distributed_trace_headers,
+        'accept_distributed_trace_headers')
+insert_distributed_trace_headers = __wrap_api_call(
+        __insert_distributed_trace_headers,
+        'insert_distributed_trace_headers')
 current_trace_id = __wrap_api_call(__current_trace_id, 'current_trace_id')
 current_span_id = __wrap_api_call(__current_span_id, 'current_span_id')
 wsgi_application = __wsgi_application
