@@ -3,6 +3,8 @@ import random
 import time
 import traceback
 from newrelic.core.trace_cache import trace_cache
+from newrelic.core.attribute import (
+        process_user_attribute, MAX_NUM_USER_ATTRIBUTES)
 
 _logger = logging.getLogger(__name__)
 
@@ -29,11 +31,17 @@ class TimeTrace(object):
         # 16-digit random hex. Padded with zeros in the front.
         self.guid = '%016x' % random.getrandbits(64)
         self.agent_attributes = {}
+        self.user_attributes = {}
 
     @property
     def transaction(self):
         return self.root and self.root.transaction
 
+    @property
+    def settings(self):
+        transaction = self.transaction
+        return transaction and transaction.settings
+
     def _is_leaf(self):
         return self.child_count == len(self.children)
 
@@ -146,6 +154,22 @@ class TimeTrace(object):
             # we may have children still running if we're async
             trace_cache().pop_current(self)
 
+    def add_custom_attribute(self, key, value):
+        settings = self.settings
+        if not settings:
+            return
+
+        if settings.high_security:
+            _logger.debug('Cannot add custom parameter in High Security Mode.')
+            return
+
+        if len(self.user_attributes) >= MAX_NUM_USER_ATTRIBUTES:
+            _logger.debug('Maximum number of custom attributes already '
+                    'added. Dropping attribute: %r=%r', key, value)
+            return
+
+        self.user_attributes[key] = value
+
     def _add_agent_attribute(self, key, value):
         self.agent_attributes[key] = value
 
@@ -306,6 +330,38 @@ class TimeTrace(object):
         else:
             self.has_async_children = False
 
+    def get_linking_metadata(self):
+        metadata = {
+            "entity.type": "SERVICE",
+        }
+        txn = self.transaction
+        if txn:
+            metadata["span.id"] = self.guid
+            metadata["trace.id"] = txn.trace_id
+            settings = txn.settings
+            if settings:
+                metadata["entity.name"] = settings.app_name
+                entity_guid = settings.entity_guid
+                if entity_guid:
+                    metadata["entity.guid"] = entity_guid
+        return metadata
+
+
+def add_custom_span_attribute(key, value):
+    trace = current_trace()
+    if trace:
+        trace.add_custom_attribute(key, value)
+
 
 def current_trace():
     return trace_cache().current_trace()
+
+
+def get_linking_metadata():
+    trace = current_trace()
+    if trace:
+        return trace.get_linking_metadata()
+    else:
+        return {
+            "entity.type": "SERVICE",
+        }
