@@ -437,6 +437,7 @@ _settings.host = os.environ.get('NEW_RELIC_HOST',
 _settings.port = int(os.environ.get('NEW_RELIC_PORT', '0'))
 
 _settings.agent_run_id = None
+_settings.entity_guid = None
 _settings.request_headers_map = {}
 
 _settings.proxy_scheme = os.environ.get('NEW_RELIC_PROXY_SCHEME', None)
@@ -513,6 +514,7 @@ _settings.encoding_key = None
 _settings.sampling_target = 10
 _settings.sampling_target_period_in_seconds = 60
 
+_settings.compressed_content_encoding = 'gzip'
 _settings.max_payload_size_in_bytes = 1000000
 
 _settings.attributes.enabled = True
@@ -532,6 +534,7 @@ _settings.custom_insights_events.enabled = True
 
 _settings.distributed_tracing.enabled = _environ_as_bool(
         'NEW_RELIC_DISTRIBUTED_TRACING_ENABLED', default=False)
+_settings.distributed_tracing.exclude_newrelic_header = False
 _settings.span_events.enabled = _environ_as_bool(
         'NEW_RELIC_SPAN_EVENTS_ENABLED', default=True)
 _settings.span_events.attributes.enabled = True
