@@ -1,18 +1,16 @@
 from collections import namedtuple
 
-import newrelic.core.attribute as attribute
 import newrelic.core.trace_node
 
 from newrelic.core.node_mixin import GenericNodeMixin
 from newrelic.core.metric import TimeMetric
-from newrelic.core.attribute_filter import DST_TRANSACTION_SEGMENTS
 
 from newrelic.packages import six
 
 _FunctionNode = namedtuple('_FunctionNode',
         ['group', 'name', 'children', 'start_time', 'end_time',
         'duration', 'exclusive', 'label', 'params', 'rollup',
-        'is_async', 'guid', 'agent_attributes'])
+        'is_async', 'guid', 'agent_attributes', 'user_attributes'])
 
 
 class FunctionNode(_FunctionNode, GenericNodeMixin):
@@ -93,18 +91,8 @@ class FunctionNode(_FunctionNode, GenericNodeMixin):
                 break
             children.append(child.trace_node(stats, root, connections))
 
-        # Agent attributes
-        params = attribute.resolve_agent_attributes(
-                self.agent_attributes,
-                root.settings.attribute_filter,
-                DST_TRANSACTION_SEGMENTS)
-
-        # User attributes override agent attributes
-        if self.params:
-            params.update(self.params)
-
-        # Intrinsic attributes override everything
-        params['exclusive_duration_millis'] = 1000.0 * self.exclusive
+        params = self.get_trace_segment_params(
+                root.settings, params=self.params)
 
         return newrelic.core.trace_node.TraceNode(start_time=start_time,
                 end_time=end_time, name=name, params=params, children=children,
