@@ -11,15 +11,15 @@ import newrelic.core.trace_node
 
 from newrelic.core.metric import ApdexMetric, TimeMetric
 from newrelic.core.string_table import StringTable
-from newrelic.core.attribute import create_user_attributes
+from newrelic.core.attribute import (create_user_attributes,
+        resolve_user_attributes, process_user_attribute)
 from newrelic.core.attribute_filter import (DST_ERROR_COLLECTOR,
-        DST_TRANSACTION_TRACER, DST_TRANSACTION_EVENTS)
-from newrelic.core.node_mixin import GenericNodeMixin
+        DST_TRANSACTION_TRACER, DST_TRANSACTION_EVENTS, DST_SPAN_EVENTS)
 
 
 _TransactionNode = namedtuple('_TransactionNode',
         ['settings', 'path', 'type', 'group', 'base_name', 'name_for_metric',
-        'port', 'request_uri', 'response_code', 'queue_start', 'start_time',
+        'port', 'request_uri', 'queue_start', 'start_time',
         'end_time', 'last_byte_time', 'response_time', 'total_time',
         'duration', 'exclusive', 'children', 'errors', 'slow_sql',
         'custom_events', 'apdex_t', 'suppress_apdex', 'custom_metrics', 'guid',
@@ -31,10 +31,11 @@ _TransactionNode = namedtuple('_TransactionNode',
         'distributed_trace_intrinsics', 'user_attributes', 'priority',
         'sampled', 'parent_transport_duration', 'parent_span', 'parent_type',
         'parent_account', 'parent_app', 'parent_tx', 'parent_transport_type',
-        'root_span_guid', 'trace_id'])
+        'root_span_guid', 'trace_id', 'loop_time', 'trusted_parent_span',
+        'tracing_vendors', 'root_span_user_attributes'])
 
 
-class TransactionNode(_TransactionNode, GenericNodeMixin):
+class TransactionNode(_TransactionNode):
 
     """Class holding data corresponding to the root of the transaction. All
     the nodes of interest recorded for the transaction are held as a tree
@@ -42,6 +43,11 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
 
     """
 
+    def __new__(cls, *args, **kwargs):
+        node = _TransactionNode.__new__(cls, *args, **kwargs)
+        node.include_transaction_trace_request_uri = False
+        return node
+
     def __hash__(self):
         return id(self)
 
@@ -57,10 +63,6 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
     def name(self):
         return self.name_for_metric
 
-    @property
-    def distributed_trace_received(self):
-        return self.trace_id != self.guid
-
     def time_metrics(self, stats):
         """Return a generator yielding the timed metrics for the
         top level web transaction as well as all the child nodes.
@@ -159,12 +161,11 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
         # Generate Distributed Tracing metrics
 
         if self.settings.distributed_tracing.enabled:
-            if self.distributed_trace_received:
-                dt_tag = "%s/%s/%s/%s/all" % (
-                    self.parent_type, self.parent_account,
-                    self.parent_app, self.parent_transport_type)
-            else:
-                dt_tag = "Unknown/Unknown/Unknown/Unknown/all"
+            dt_tag = "%s/%s/%s/%s/all" % (
+                self.parent_type or 'Unknown',
+                self.parent_account or 'Unknown',
+                self.parent_app or 'Unknown',
+                self.parent_transport_type or 'Unknown')
 
             for bonus_tag in ('', metric_suffix):
                 yield TimeMetric(
@@ -173,7 +174,7 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
                     duration=self.duration,
                     exclusive=self.duration)
 
-                if self.distributed_trace_received:
+                if self.parent_transport_duration is not None:
                     yield TimeMetric(
                         name="TransportDuration/%s%s" % (dt_tag, bonus_tag),
                         scope='',
@@ -290,7 +291,6 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
 
         for error in self.errors:
             params = {}
-            params["request_uri"] = self.request_uri
             params["stack_trace"] = error.stack_trace
 
             params['intrinsics'] = self.trace_intrinsics
@@ -336,7 +336,11 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
                 break
             children.append(child.trace_node(stats, root, connections))
 
-        params = {}
+        params = resolve_user_attributes(
+                self.processed_span_user_attributes,
+                self.settings.attribute_filter,
+                DST_TRANSACTION_TRACER)
+
         params['exclusive_duration_millis'] = 1000.0 * self.exclusive
 
         return newrelic.core.trace_node.TraceNode(
@@ -364,6 +368,8 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
         for attr in self.agent_attributes:
             if attr.destinations & DST_TRANSACTION_TRACER:
                 attributes['agentAttributes'][attr.name] = attr.value
+                if attr.name == 'request.uri':
+                    self.include_transaction_trace_request_uri = True
 
         attributes['userAttributes'] = {}
         for attr in self.user_attributes:
@@ -536,7 +542,7 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
 
         intrinsics = self.distributed_trace_intrinsics.copy()
 
-        intrinsics['timestamp'] = self.start_time
+        intrinsics['timestamp'] = int(1000.0 * self.start_time)
         intrinsics['duration'] = self.response_time
 
         if self.port:
@@ -578,22 +584,50 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
         _add_call_count('External/all', 'externalCallCount')
         _add_call_count('Datastore/all', 'databaseCallCount')
 
+        if self.loop_time:
+            intrinsics['eventLoopTime'] = self.loop_time
+        _add_call_time('EventLoop/Wait/all', 'eventLoopWait')
+
         self._event_intrinsics_cache = intrinsics.copy()
 
         return intrinsics
 
-    def span_event(self, *args, **kwargs):
-        attrs = super(TransactionNode, self).span_event(*args, **kwargs)
-        i_attrs = attrs[0]
+    @property
+    def processed_span_user_attributes(self):
+        if hasattr(self, '_processed_span_user_attribtes'):
+            return self._processed_span_user_attribtes
 
-        # GUID needs to come from Sentinel for the root span event since that
-        # is what's forwarded in the distributed trace payload.
+        self._processed_span_user_attributes = u_attrs = {}
+        for k, v in self.root_span_user_attributes.items():
+            k, v = process_user_attribute(k,v)
+            u_attrs[k] = v
+        return u_attrs
+
+    def span_event(self, settings, base_attrs=None, parent_guid=None):
+        i_attrs = base_attrs and base_attrs.copy() or {}
+        i_attrs['type'] = 'Span'
+        i_attrs['name'] = self.name
         i_attrs['guid'] = self.root_span_guid
+        i_attrs['timestamp'] = int(self.start_time * 1000)
+        i_attrs['duration'] = self.duration
+        i_attrs['category'] = 'generic'
         i_attrs['nr.entryPoint'] = True
 
-        return attrs
+        if parent_guid:
+            i_attrs['parentId'] = parent_guid
+        if self.trusted_parent_span:
+            i_attrs['trustedParentId'] = self.trusted_parent_span
+        if self.tracing_vendors:
+            i_attrs['tracingVendors'] = self.tracing_vendors
 
-    def span_events(self, stats):
+        u_attrs = resolve_user_attributes(
+                self.processed_span_user_attributes,
+                settings.attribute_filter,
+                DST_SPAN_EVENTS)
+
+        return [i_attrs, u_attrs, {}]
+
+    def span_events(self, settings):
         base_attrs = {
             'transactionId': self.guid,
             'traceId': self.trace_id,
@@ -601,12 +635,12 @@ class TransactionNode(_TransactionNode, GenericNodeMixin):
             'priority': self.priority,
         }
 
-        yield self.span_event(base_attrs,
+        yield self.span_event(settings, base_attrs,
                 parent_guid=self.parent_span)
 
         for child in self.children:
             for event in child.span_events(
-                    stats,
+                    settings,
                     base_attrs,
                     parent_guid=self.root_span_guid):
                 yield event
