@@ -29,7 +29,7 @@ def _add_consume_rabbitmq_trace(transaction, method, properties,
     reply_to = properties.get('reply_to')
     headers = properties.get('headers')
 
-    # Delete CAT headers
+    # Do not record dt headers in the segment parameters
     if headers:
         headers.pop(
                 MessageTrace.cat_id_key, None)
@@ -37,6 +37,8 @@ def _add_consume_rabbitmq_trace(transaction, method, properties,
                 MessageTrace.cat_transaction_key, None)
         headers.pop(
                 MessageTrace.cat_distributed_trace_key, None)
+        headers.pop('traceparent', None)
+        headers.pop('tracestate', None)
 
     # The transaction may have started after the message was received. In this
     # case, the start time is reset to the true transaction start time.
@@ -56,7 +58,7 @@ def _add_consume_rabbitmq_trace(transaction, method, properties,
         params['queue_name'] = queue_name
 
     # create a trace starting at the time the message was received
-    trace = MessageTrace(transaction, library='RabbitMQ',
+    trace = MessageTrace(library='RabbitMQ',
             operation='Consume',
             destination_type='Exchange',
             destination_name=method.exchange or 'Default',
@@ -66,13 +68,9 @@ def _add_consume_rabbitmq_trace(transaction, method, properties,
     trace.__exit__(None, None, None)
 
 
-def _bind_basic_publish(exchange, routing_key, body,
-                    properties=None, mandatory=False, immediate=False):
-    return (exchange, routing_key, body, properties, mandatory, immediate)
-
-
-def _bind_params_basic_get(callback=None, queue='', *args, **kwargs):
-    return callback, queue
+def _bind_basic_publish(
+        exchange, routing_key, body, properties=None, *args, **kwargs):
+    return (exchange, routing_key, body, properties, args, kwargs)
 
 
 def _nr_wrapper_basic_publish(wrapped, instance, args, kwargs):
@@ -83,25 +81,20 @@ def _nr_wrapper_basic_publish(wrapped, instance, args, kwargs):
 
     from pika import BasicProperties
 
-    (exchange, routing_key, body, properties, mandatory, immediate) = (
+    (exchange, routing_key, body, properties, args, kwargs) = (
             _bind_basic_publish(*args, **kwargs))
     properties = properties or BasicProperties()
     properties.headers = properties.headers or {}
     user_headers = properties.headers.copy()
-    cat_headers = MessageTrace.generate_request_headers(transaction)
 
-    for name, value in cat_headers:
-        properties.headers[name] = value
+    # Do not record dt headers in the segment parameters
+    user_headers.pop(MessageTrace.cat_id_key, None)
+    user_headers.pop(MessageTrace.cat_transaction_key, None)
+    user_headers.pop(MessageTrace.cat_distributed_trace_key, None)
+    user_headers.pop("traceparent", None)
+    user_headers.pop("tracestate", None)
 
-    # Do not record cat headers in the segment parameters
-    if MessageTrace.cat_id_key in user_headers:
-        del user_headers[MessageTrace.cat_id_key]
-    if MessageTrace.cat_transaction_key in user_headers:
-        del user_headers[MessageTrace.cat_transaction_key]
-    if MessageTrace.cat_distributed_trace_key in user_headers:
-        del user_headers[MessageTrace.cat_distributed_trace_key]
-
-    args = (exchange, routing_key, body, properties, mandatory, immediate)
+    args = (exchange, routing_key, body, properties) + args
 
     params = {}
     if routing_key is not None:
@@ -113,52 +106,48 @@ def _nr_wrapper_basic_publish(wrapped, instance, args, kwargs):
     if user_headers:
         params['headers'] = user_headers
 
-    with MessageTrace(transaction, library='RabbitMQ',
+    with MessageTrace(library='RabbitMQ',
             operation='Produce',
             destination_type='Exchange',
             destination_name=exchange or 'Default',
             params=params):
-        return wrapped(*args)
-
-
-def _nr_wrapper_basic_get(wrapped, instance, args, kwargs):
-
-    callback, queue = _bind_params_basic_get(*args, **kwargs)
-    if callback is None:
+        cat_headers = MessageTrace.generate_request_headers(transaction)
+        properties.headers.update(cat_headers)
         return wrapped(*args, **kwargs)
 
-    @function_wrapper
-    def callback_wrapper(_wrapped, _instance, _args, _kwargs):
-        transaction = current_transaction()
 
-        if transaction is None:
-            return callback(*_args, **_kwargs)
+def _wrap_Channel_get_callback(module, obj, wrap_get):
+    def _nr_wrapper_basic_get(wrapped, instance, args, kwargs):
 
-        if not _kwargs:
-            method, properties = _args[1:3]
-            start_time = getattr(callback_wrapper, '_nr_start_time', None)
+        @function_wrapper
+        def callback_wrapper(callback, _instance, _args, _kwargs):
+            transaction = current_transaction()
 
-            _add_consume_rabbitmq_trace(transaction,
-                    method=method,
-                    properties=properties,
-                    nr_start_time=start_time,
-                    queue_name=queue)
-        else:
-            m = transaction._transaction_metrics.get(KWARGS_ERROR, 0)
-            transaction._transaction_metrics[KWARGS_ERROR] = m + 1
+            if transaction is None:
+                return callback(*_args, **_kwargs)
 
-        name = callable_name(callback)
-        with FunctionTrace(transaction=transaction, name=name):
-            return callback(*_args, **_kwargs)
+            if not _kwargs:
+                method, properties = _args[1:3]
+                start_time = getattr(callback_wrapper, '_nr_start_time', None)
 
-    callback_wrapper._nr_start_time = time.time()
-    if len(args) > 0:
-        args = list(args)
-        args[0] = callback_wrapper(callback)
-    else:
-        kwargs['callback'] = callback_wrapper(callback)
+                _add_consume_rabbitmq_trace(transaction,
+                        method=method,
+                        properties=properties,
+                        nr_start_time=start_time,
+                        queue_name=queue)
+            else:
+                m = transaction._transaction_metrics.get(KWARGS_ERROR, 0)
+                transaction._transaction_metrics[KWARGS_ERROR] = m + 1
 
-    return wrapped(*args, **kwargs)
+            name = callable_name(callback)
+            with FunctionTrace(name=name):
+                return callback(*_args, **_kwargs)
+
+        callback_wrapper._nr_start_time = time.time()
+        queue, args, kwargs = wrap_get(callback_wrapper, *args, **kwargs)
+        return wrapped(*args, **kwargs)
+
+    wrap_function_wrapper(module, obj, _nr_wrapper_basic_get)
 
 
 def _nr_wrapper_Basic_Deliver_init_(wrapper, instance, args, kwargs):
@@ -176,14 +165,34 @@ def _nr_wrap_BlockingChannel___init__(wrapped, instance, args, kwargs):
     return ret
 
 
-def _bind_params_BlockingChannel_basic_consume(consumer_callback, queue, *args,
-        **kwargs):
-    return consumer_callback, queue
+def _wrap_basic_consume_BlockingChannel_old(wrapper,
+        consumer_callback, queue, *args, **kwargs):
+    args = (wrapper(consumer_callback), queue) + args
+    return queue, args, kwargs
 
 
-def _bind_params_Channel_basic_consume(consumer_callback, queue='', *args,
+def _wrap_basic_consume_Channel_old(wrapper, consumer_callback, queue='',
+        *args, **kwargs):
+    return queue, (wrapper(consumer_callback), queue) + args, kwargs
+
+
+def _wrap_basic_consume_Channel(wrapper, queue, on_message_callback, *args,
         **kwargs):
-    return consumer_callback, queue
+    args = (queue, wrapper(on_message_callback)) + args
+    return queue, args, kwargs
+
+
+def _wrap_basic_get_Channel(wrapper, queue, callback, *args, **kwargs):
+    args = (queue, wrapper(callback)) + args
+    return queue, args, kwargs
+
+
+def _wrap_basic_get_Channel_old(wrapper, callback=None, queue='',
+        *args, **kwargs):
+    if callback is not None:
+        callback = wrapper(callback)
+    args = (callback, queue) + args
+    return queue, args, kwargs
 
 
 def _ConsumeGeneratorWrapper(wrapped):
@@ -265,7 +274,7 @@ def _ConsumeGeneratorWrapper(wrapped):
                     else:
                         yielded = generator.send(value)
 
-                    if yielded:
+                    if yielded is not None and any(yielded):
                         created_bt = _possibly_create_traces(yielded)
 
                     try:
@@ -276,9 +285,10 @@ def _ConsumeGeneratorWrapper(wrapped):
                     if created_bt:
                         created_bt.__exit__(*exc)
 
-            except (GeneratorExit, StopIteration):
+            except GeneratorExit:
                 raise
-
+            except StopIteration:
+                pass
             except Exception:
                 exc = sys.exc_info()
                 raise
@@ -301,27 +311,27 @@ def _ConsumeGeneratorWrapper(wrapped):
     return FunctionWrapper(wrapped, wrapper)
 
 
-def _wrap_Channel_consume_callback(module, obj, bind_params,
-        callback_referrer):
+def _wrap_Channel_consume_callback(module, obj, wrap_consume):
 
     @function_wrapper
-    def _nr_wrapper_Channel_consume_(wrapped, instance, args, kwargs):
-        callback, queue = bind_params(*args, **kwargs)
-        name = callable_name(callback)
+    def _nr_wrapper_Channel_consume_(wrapped, channel, args, kwargs):
 
         @function_wrapper
-        def callback_wrapper(_wrapped, _instance, _args, _kwargs):
+        def callback_wrapper(wrapped, instance, args, kwargs):
+            name = callable_name(wrapped)
+
             transaction = current_transaction(active_only=False)
 
             if transaction and (transaction.ignore_transaction or
                     transaction.stopped):
-                return callback(*_args, **_kwargs)
+                return wrapped(*args, **kwargs)
             elif transaction:
-                with FunctionTrace(transaction=transaction, name=name):
-                    return callback(*_args, **_kwargs)
+                with FunctionTrace(name=name):
+                    return wrapped(*args, **kwargs)
             else:
-                if hasattr(instance, '_nr_disable_txn_tracing'):
-                    return callback(*_args, **_kwargs)
+                if hasattr(channel, '_nr_disable_txn_tracing'):
+                    return wrapped(*args, **kwargs)
+
                 # Keyword arguments are unknown since this is a user
                 # defined callback
                 exchange = 'Unknown'
@@ -330,8 +340,8 @@ def _wrap_Channel_consume_callback(module, obj, bind_params,
                 reply_to = None
                 correlation_id = None
                 unknown_kwargs = False
-                if not _kwargs:
-                    method, properties = _args[1:3]
+                if not kwargs:
+                    method, properties = args[1:3]
                     exchange = method.exchange or 'Default'
                     routing_key = getattr(method, 'routing_key', None)
                     if properties is not None:
@@ -363,14 +373,9 @@ def _wrap_Channel_consume_callback(module, obj, bind_params,
                         m = mt._transaction_metrics.get(KWARGS_ERROR, 0)
                         mt._transaction_metrics[KWARGS_ERROR] = m + 1
 
-                    return _wrapped(*_args, **_kwargs)
-
-        if len(args) > 0:
-            args = list(args)
-            args[0] = callback_wrapper(callback)
-        else:
-            kwargs[callback_referrer] = callback_wrapper(callback)
+                    return wrapped(*args, **kwargs)
 
+        queue, args, kwargs = wrap_consume(callback_wrapper, *args, **kwargs)
         return wrapped(*args, **kwargs)
 
     # Normally, wrap_object(module, object, ...) would be used here.
@@ -390,10 +395,18 @@ def _disable_channel_transactions(wrapped, instance, args, kwargs):
 
 
 def instrument_pika_adapters(module):
-    _wrap_Channel_consume_callback(module.blocking_connection,
+    import pika
+    version = tuple(int(num) for num in pika.__version__.split('.', 1)[0])
+
+    if version[0] < 1:
+        wrap_consume = _wrap_basic_consume_BlockingChannel_old
+    else:
+        wrap_consume = _wrap_basic_consume_Channel
+
+    _wrap_Channel_consume_callback(
+            module.blocking_connection,
             'BlockingChannel.basic_consume',
-            _bind_params_BlockingChannel_basic_consume,
-            'consumer_callback')
+            wrap_consume)
     wrap_function_wrapper(module.blocking_connection,
             'BlockingChannel.__init__', _nr_wrap_BlockingChannel___init__)
     wrap_object(module.blocking_connection, 'BlockingChannel.consume',
@@ -410,10 +423,21 @@ def instrument_pika_spec(module):
 
 
 def instrument_pika_channel(module):
+    import pika
+    version = tuple(int(num) for num in pika.__version__.split('.', 1)[0])
+
+    if version[0] < 1:
+        wrap_consume = _wrap_basic_consume_Channel_old
+        wrap_get = _wrap_basic_get_Channel_old
+    else:
+        wrap_consume = _wrap_basic_consume_Channel
+        wrap_get = _wrap_basic_get_Channel
+
     wrap_function_wrapper(module, 'Channel.basic_publish',
             _nr_wrapper_basic_publish)
-    wrap_function_wrapper(module, 'Channel.basic_get',
-            _nr_wrapper_basic_get)
 
-    _wrap_Channel_consume_callback(module, 'Channel.basic_consume',
-            _bind_params_Channel_basic_consume, 'consumer_callback')
+    _wrap_Channel_get_callback(module, 'Channel.basic_get', wrap_get)
+    _wrap_Channel_consume_callback(
+            module,
+            'Channel.basic_consume',
+            wrap_consume)
